@@ -1,6 +1,7 @@
 import numpy as np
 from numba import cuda
 from numba import jit
+from .fs import FocalSet
     
 class Lattice:
     
@@ -12,11 +13,12 @@ class Lattice:
         :param lattice: [description]
         :type lattice: [type]
         """
+        
         self.fod = fod
         self.b = None # array to store the belief
         
         # TODO check that ds is associated with the same fod object
-        if fs != None:
+        if isinstance(fs, (FocalSet)):
             if fs.fod != fod:
                 raise ValueError('The focal set and this lattice must be associated to the same the frame of discernment' )
             else:
@@ -43,23 +45,34 @@ class Lattice:
         :rtype: np.array
         """
         
-        if self.fs == None:
+        if not isinstance(self.fs, (FocalSet)):
             raise ValueError("A focal set must be added to the lattice before computing the belief.")
         
         if cuda.is_available():
             if isinstance(self.b, (np.ndarray)):
                 return self.b
             else:
+                # Initialize the array to store the belief of each subset
+                self.b = np.zeros(2^self.fod.n)
+                
+                # Define the dimensions of the parallelization
                 threadsperblock = 128 # TODO which one is the best to use?
                 blockspergrid = self.b.size / threadsperblock
-                # focal set object
+                
+                # Get the focal set as two diffent arrays
                 elements, bpas = self.fs.as_arrays()
+                
+                # Copy to the GPU
                 d_elements = cuda.to_device(elements)
-                d_bpa = cuda.to_device(bpas)
-                self.b = np.zeros(2^self.fod.n)
+                d_bpa = cuda.to_device(bpas)     
                 d_b = cuda.to_device(self.b) # TODO is it better to initialize in gpu ? 
+                
                 self.__bel[blockspergrid, threadsperblock](d_elements, d_bpa, d_b)
+                
+                # Get the results back
                 d_b.copy_to_host(self.b)
+                # TODO should I clean somehow the GPU?
+                
                 return self.b
         else:
             raise ValueError("No GPU available.")
